@@ -1,7 +1,7 @@
 /* fuse.h: Variables exported from the main file
    Copyright (c) 2000 Philip Kendall
 
-   $Id: fuse.h,v 1.9 2002/01/12 14:01:26 pak21 Exp $
+   $Id: fuse.h,v 1.10 2002/03/21 11:58:05 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ extern char* fuse_progname;		/* argv[0] */
 
 extern int fuse_exiting;		/* Shall we exit now? */
 
-extern int fuse_emulation_running;	/* Is Spectrum emulation running? */
+extern int fuse_emulation_paused;	/* Is Spectrum emulation paused? */
 int fuse_emulation_pause(void);		/* Stop and start emulation */
 int fuse_emulation_unpause(void);
 
