@@ -1,7 +1,7 @@
 /* machine.c: Routines for handling the various machine types
-   Copyright (c) 1999-2001 Philip Kendall
+   Copyright (c) 1999-2002 Philip Kendall
 
-   $Id: machine.c,v 1.24 2002/01/23 19:02:06 pak21 Exp $
+   $Id: machine.c,v 1.31 2002/03/21 21:53:36 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include <config.h>
 
 #include <assert.h>
+#include <errno.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
@@ -43,10 +44,10 @@
 #include "specplus2.h"
 #include "specplus3.h"
 #include "tape.h"
+#include "ui/ui.h"
 #include "utils.h"
 #include "z80/z80.h"
 
-#define ERROR_MESSAGE_MAX_LENGTH 1024
 #define PATHNAME_MAX_LENGTH 1024
 
 machine_info **machine_types = NULL; /* Array of available machines */
@@ -86,16 +87,14 @@ static int machine_add_machine( int (*init_function)( machine_info *machine ) )
     (machine_info**)realloc( machine_types, 
 			     machine_count * sizeof( machine_info* ) );
   if( machine_types == NULL ) {
-    fprintf(stderr, "%s: out of memory at %s:%d\n", fuse_progname,
-	    __FILE__, __LINE__ );
+    ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
     return 1;
   }
 
   machine_types[ machine_count - 1 ] =
     (machine_info*)malloc( sizeof( machine_info ) );
   if( machine_types[ machine_count - 1 ] == NULL ) {
-    fprintf(stderr, "%s: out of memory at %s:%d\n", fuse_progname,
-	    __FILE__, __LINE__ );
+    ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
     return 1;
   }
 
@@ -119,8 +118,7 @@ int machine_select( int type )
     }
   }
 
-  fprintf( stderr, "%s: Machine type `%d' unavailable\n", fuse_progname,
-	   type );
+  ui_error( UI_ERROR_ERROR, "Machine type '%d' unavailable", type );
   return 1;
 }
 
@@ -136,7 +134,7 @@ int machine_select_id( const char *id )
     }
   }
 
-  fprintf( stderr, "%s: Machine id `%s' unknown\n", fuse_progname, id );
+  ui_error( UI_ERROR_ERROR, "Machine id '%s' unknown", id );
   return 1;
 }
 
@@ -207,8 +205,14 @@ int machine_allocate_roms( machine_info *machine, size_t count )
 
   machine->roms = (BYTE**)malloc( count * sizeof(BYTE*) );
   if( machine->roms == NULL ) {
-    fprintf(stderr, "%s: out of memory at %s:%d\n", fuse_progname,
-	    __FILE__, __LINE__ );
+    ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
+    return 1;
+  }
+
+  machine->rom_lengths = (size_t*)malloc( count * sizeof(size_t) );
+  if( machine->rom_lengths == NULL ) {
+    ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
+    free( machine->roms );
     return 1;
   }
 
@@ -218,7 +222,7 @@ int machine_allocate_roms( machine_info *machine, size_t count )
 int machine_read_rom( machine_info *machine, size_t number,
 		      const char* filename )
 {
-  int fd; size_t length;
+  int fd;
 
   int error;
 
@@ -226,34 +230,44 @@ int machine_read_rom( machine_info *machine, size_t number,
 
   fd = machine_find_rom( filename );
   if( fd == -1 ) {
-    fprintf( stderr, "%s: couldn't find ROM `%s'\n", fuse_progname, filename );
+    ui_error( UI_ERROR_ERROR, "couldn't find ROM '%s'", filename );
     return 1;
   }
 
-  error = utils_read_fd( fd, filename, &(machine->roms[number]), &length );
+  error = utils_read_fd( fd, filename, &(machine->roms[number]),
+			 &(machine->rom_lengths[number]) );
   if( error ) return error;
 
   return 0;
 
 }
 
-/* Find a ROM called `filename'; look in the current directory, ./roms
-   and the defined roms directory; returns a fd for the ROM on success,
-   -1 if it couldn't find the ROM */
+/* Find a ROM called `filename' in some likely locations; returns a fd
+   for the ROM on success or -1 if it couldn't find the ROM */
 int machine_find_rom( const char *filename )
 {
   int fd;
 
   char path[ PATHNAME_MAX_LENGTH ];
 
+  /* First look off the current directory */
   snprintf( path, PATHNAME_MAX_LENGTH, "roms/%s", filename );
   fd = open( path, O_RDONLY );
   if( fd != -1 ) return fd;
 
+  /* Then look where Fuse may have installed the ROMs */
   snprintf( path, PATHNAME_MAX_LENGTH, "%s/%s", DATADIR, filename );
   fd = open( path, O_RDONLY );
   if( fd != -1 ) return fd;
 
+#ifdef ROMSDIR
+  /* Finally look in a system-wide directory for ROMs. Debian uses
+     /usr/share/spectrum-roms/ here */
+  snprintf( path, PATHNAME_MAX_LENGTH, "%s/%s", ROMSDIR, filename );
+  fd = open( path, O_RDONLY );
+  if( fd != -1 ) return fd;
+#endif
+
   return -1;
 }
 
@@ -277,16 +291,11 @@ static int machine_free_machine( machine_info *machine )
 {
   size_t i;
 
-  char error_message[ ERROR_MESSAGE_MAX_LENGTH ];
-
   for( i=0; i<machine->rom_count; i++ ) {
 
-    /* FIXME: should carry the length through properly */
-    if( munmap( machine->roms[i], 0x4000 ) == -1 ) {
-      snprintf( error_message, ERROR_MESSAGE_MAX_LENGTH,
-		"%s: couldn't munmap ROM %lu",
-		fuse_progname, (unsigned long)i );
-      perror( error_message );
+    if( munmap( machine->roms[i], machine->rom_lengths[i] ) == -1 ) {
+      ui_error( UI_ERROR_ERROR, "couldn't munmap ROM %lu: %s",
+		(unsigned long)i, strerror( errno ) );
       return 1;
     }
   }
