@@ -1,5 +1,5 @@
 /* sound.c: Sound support
-   Copyright (c) 2000-2001 Russell Marks, Matan Ziv-Av, Philip Kendall
+   Copyright (c) 2000-2002 Russell Marks, Matan Ziv-Av, Philip Kendall
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,6 +33,8 @@
  * very high at all. And I speak as a Cyrix owner. :-)
  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -42,17 +44,14 @@
 #include <sys/ioctl.h>
 #include <fcntl.h>
 
-#include <config.h>
-
-#if defined(HAVE_SYS_SOUNDCARD_H)
-#include "osssound.h"
-#endif
-
 #include "fuse.h"
 #include "machine.h"
+#include "osssound.h"
 #include "settings.h"
 #include "sound.h"
 #include "spectrum.h"
+#include "sunsound.h"
+#include "ui/ui.h"
 
 /* configuration */
 int sound_enabled=0;		/* Are we currently using the sound card;
@@ -166,21 +165,20 @@ ay_change_count=0;
 }
 
 
-void sound_init(void)
+void sound_init(const char *device)
 {
 static int first_init=1;
 int f,ret;
 
 /* if we don't have any sound I/O code compiled in, don't do sound */
-#if !defined(HAVE_SYS_SOUNDCARD_H)	/* only type for now */
+#if !defined(HAVE_SYS_SOUNDCARD_H) && !defined(HAVE_SYS_AUDIOIO_H)
 return;
 #endif
 
 if(sound_enabled)
   {
-  fprintf(stderr,
-          "%s: sound_init() called with sound_enabled set, can't happen!\n",
-          fuse_progname);
+  ui_error(UI_ERROR_ERROR,
+	   "sound_init() called with sound_enabled set, can't happen!" );
   sound_end();
   return;
   }
@@ -192,7 +190,10 @@ if(sound_stereo_ay || sound_stereo_beeper)
   sound_stereo=1;
 
 #if defined(HAVE_SYS_SOUNDCARD_H)
-ret=osssound_init(&sound_freq,&sound_stereo);
+ret=osssound_init(device,&sound_freq,&sound_stereo);
+#endif
+#if defined(HAVE_SYS_AUDIOIO_H)
+ret=sunsound_init(device,&sound_freq,&sound_stereo);
 #endif
 
 if(ret)
@@ -288,6 +289,9 @@ if(sound_enabled)
     free(sound_buf);
 #if defined(HAVE_SYS_SOUNDCARD_H)
   osssound_end();
+#endif
+#if defined(HAVE_SYS_AUDIOIO_H)
+  sunsound_end();
 #endif
   sound_enabled=0;
   }
@@ -754,6 +758,9 @@ sound_ay_overlay();
 #if defined(HAVE_SYS_SOUNDCARD_H)
 osssound_frame(sound_buf,sound_framesiz*sound_channels);
 #endif
+#if defined(HAVE_SYS_AUDIOIO_H)
+sunsound_frame(sound_buf,sound_framesiz*sound_channels);
+#endif
 
 sound_oldpos[0]=sound_oldpos[1]=-1;
 sound_fillpos[0]=sound_fillpos[1]=0;
