@@ -1,7 +1,7 @@
 /* tape.h: tape handling routines
    Copyright (c) 1999-2001 Philip Kendall
 
-   $Id: tape.h,v 1.10 2001/12/24 17:35:01 pak21 Exp $
+   $Id: tape.h,v 1.11 2002/02/24 13:25:16 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +27,10 @@
 #ifndef FUSE_TAPE_H
 #define FUSE_TAPE_H
 
+#ifndef FUSE_TYPES_H
+#include "types.h"
+#endif
+
 int tape_init( void );
 
 int tape_open( const char *filename );
@@ -40,7 +44,7 @@ int tape_save_trap( void );
 int tape_play( void );
 int tape_toggle_play( void );
 
-int tape_next_edge( void );
+int tape_next_edge( DWORD last_tstates );
 
 int tape_stop( void );
 
