@@ -1,7 +1,7 @@
 /* gtkui.c: GTK+ routines for dealing with the user interface
-   Copyright (c) 2000-2001 Philip Kendall, Russell Marks
+   Copyright (c) 2000-2002 Philip Kendall, Russell Marks
 
-   $Id: gtkui.c,v 1.36 2002/01/31 00:02:00 pak21 Exp $
+   $Id: gtkui.c,v 1.48 2002/03/21 21:53:37 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,11 +38,15 @@
 #include "display.h"
 #include "fuse.h"
 #include "gtkkeyboard.h"
+#include "gtkui.h"
 #include "machine.h"
+#include "options.h"
+#include "rzx.h"
 #include "settings.h"
 #include "snapshot.h"
 #include "spectrum.h"
 #include "tape.h"
+#include "timer.h"
 #include "ui/ui.h"
 #include "ui/uidisplay.h"
 #include "widget/widget.h"
@@ -53,6 +57,9 @@ GtkWidget *gtkui_window;
 /* The area into which the screen will be drawn */
 GtkWidget *gtkui_drawing_area;
 
+/* Popup menu widget(s), as invoked by F1 */
+GtkWidget *gtkui_menu_popup;
+
 static gboolean gtkui_make_menu(GtkAccelGroup **accel_group,
 				GtkWidget **menu_bar,
 				GtkItemFactoryEntry *menu_data,
@@ -62,8 +69,12 @@ static gint gtkui_delete(GtkWidget *widget, GdkEvent *event,
 			      gpointer data);
 static void gtkui_open(GtkWidget *widget, gpointer data);
 static void gtkui_save(GtkWidget *widget, gpointer data);
+
+static void gtkui_rzx_start( GtkWidget *widget, gpointer data );
+static void gtkui_rzx_stop( GtkWidget *widget, gpointer data );
+static void gtkui_rzx_play( GtkWidget *widget, gpointer data );
+
 static void gtkui_quit(GtkWidget *widget, gpointer data);
-static void gtkui_options( GtkWidget *widget, gpointer data );
 static void gtkui_reset(GtkWidget *widget, gpointer data);
 
 static void gtkui_select(GtkWidget *widget, gpointer data);
@@ -77,22 +88,24 @@ static void gtkui_tape_write( GtkWidget *widget, gpointer data );
 
 static void gtkui_help_keyboard( GtkWidget *widget, gpointer data );
 
-static void gtkui_destroy_widget_and_quit( GtkWidget *widget, gpointer data );
-
-static char* gtkui_fileselector_get_filename( void );
+static char* gtkui_fileselector_get_filename( const char *title );
 static void gtkui_fileselector_done( GtkButton *button, gpointer user_data );
 static void gtkui_fileselector_cancel( GtkButton *button, gpointer user_data );
 
-static void gtkui_options_done( GtkWidget *widget, gpointer user_data );
-
 static GtkItemFactoryEntry gtkui_menu_data[] = {
   { "/File",		        NULL , NULL,                0, "<Branch>"    },
   { "/File/_Open Snapshot..." , "F3" , gtkui_open,          0, NULL          },
   { "/File/_Save Snapshot..." , "F2" , gtkui_save,          0, NULL          },
   { "/File/separator",          NULL , NULL,                0, "<Separator>" },
+  { "/File/_Recording",		NULL , NULL,		    0, "<Branch>"    },
+  { "/File/Recording/_Record...",NULL, gtkui_rzx_start,     0, NULL	     },
+  { "/File/Recording/_Play...", NULL , gtkui_rzx_play,	    0, NULL          },
+  { "/File/Recording/_Stop",    NULL , gtkui_rzx_stop,	    0, NULL          },
+  { "/File/separator",          NULL , NULL,                0, "<Separator>" },
   { "/File/E_xit",	        "F10", gtkui_quit,          0, NULL          },
   { "/Options",		        NULL , NULL,                0, "<Branch>"    },
-  { "/Options/_General...",     "F4" , gtkui_options,       0, NULL          },
+  { "/Options/_General...",     "F4" , gtkoptions_general,  0, NULL          },
+  { "/Options/_Sound...",	NULL , gtkoptions_sound,    0, NULL          },
   { "/Machine",		        NULL , NULL,                0, "<Branch>"    },
   { "/Machine/_Reset",	        "F5" , gtkui_reset,         0, NULL          },
   { "/Machine/_Select...",      "F9" , gtkui_select,        0, NULL          },
@@ -105,7 +118,8 @@ static GtkItemFactoryEntry gtkui_menu_data[] = {
   { "/Help",			NULL , NULL,		    0, "<Branch>"    },
   { "/Help/_Keyboard...",	NULL , gtkui_help_keyboard, 0, NULL	     },
 };
-static guint gtkui_menu_data_size = 18;
+static guint gtkui_menu_data_size =
+  sizeof( gtkui_menu_data ) / sizeof( GtkItemFactoryEntry );
   
 int ui_init(int *argc, char ***argv, int width, int height)
 {
@@ -189,13 +203,31 @@ static gboolean gtkui_make_menu(GtkAccelGroup **accel_group,
   *accel_group = gtk_accel_group_new();
   item_factory = gtk_item_factory_new( GTK_TYPE_MENU_BAR, "<main>",
 				       *accel_group );
-
   gtk_item_factory_create_items(item_factory, menu_data_size, menu_data, NULL);
   *menu_bar = gtk_item_factory_get_widget( item_factory, "<main>" );
 
+  /* We have to recreate the menus for the popup, unfortunately... */
+  item_factory = gtk_item_factory_new( GTK_TYPE_MENU, "<main>", NULL );
+  gtk_item_factory_create_items(item_factory, menu_data_size, menu_data, NULL);
+  gtkui_menu_popup = gtk_item_factory_get_widget( item_factory, "<main>" );
+
   return FALSE;
 }
 
+static void gtkui_popup_menu_pos( GtkMenu *menu, gint *xp, gint *yp,
+				  GtkWidget *data)
+{
+  gdk_window_get_position( gtkui_window->window, xp, yp );
+}
+
+/* Popup main menu, as invoked by F1. */
+void gtkui_popup_menu(void)
+{
+  gtk_menu_popup( GTK_MENU(gtkui_menu_popup), NULL, NULL,
+		  (GtkMenuPositionFunc)gtkui_popup_menu_pos, NULL,
+		  0, 0 );
+}
+
 int ui_event(void)
 {
   while(gtk_events_pending())
@@ -239,7 +271,7 @@ static void gtkui_open(GtkWidget *widget, gpointer data)
 
   fuse_emulation_pause();
 
-  filename = gtkui_fileselector_get_filename();
+  filename = gtkui_fileselector_get_filename( "Fuse - Load Snapshot" );
   if( !filename ) { fuse_emulation_unpause(); return; }
 
   snapshot_read( filename );
@@ -258,7 +290,7 @@ static void gtkui_save(GtkWidget *widget, gpointer data)
 
   fuse_emulation_pause();
 
-  filename = gtkui_fileselector_get_filename();
+  filename = gtkui_fileselector_get_filename( "Fuse - Save Snapshot" );
   if( !filename ) { fuse_emulation_unpause(); return; }
 
   snapshot_write( filename );
@@ -268,113 +300,75 @@ static void gtkui_save(GtkWidget *widget, gpointer data)
   fuse_emulation_unpause();
 }
 
+/* Called when File/Recording/Record selected */
+static void gtkui_rzx_start( GtkWidget *widget, gpointer data )
+{
+  char *snapshot, *recording;
+
+  if( rzx_playback || rzx_recording ) return;
+
+  fuse_emulation_pause();
+
+  snapshot = gtkui_fileselector_get_filename( "Fuse - Recording Snapshot" );
+  if( !snapshot ) { fuse_emulation_unpause(); return; }
+  
+  recording = gtkui_fileselector_get_filename( "Fuse - Start Recording" );
+  if( !recording ) { free( snapshot ); fuse_emulation_unpause(); return; }
+
+  if( snapshot_write( snapshot ) ) {
+    free( snapshot ); free( recording );
+    return;
+  }
+
+  rzx_start_recording( recording );
+
+  free( snapshot ); free( recording );
+
+  fuse_emulation_unpause();
+}
+
+/* Called when File/Recording/Stop selected */
+static void gtkui_rzx_stop( GtkWidget *widget, gpointer data )
+{
+  if( rzx_recording ) rzx_stop_recording();
+  if( rzx_playback  ) rzx_stop_playback();
+}
+
+/* Called when File/Recording/Play selected */
+static void gtkui_rzx_play( GtkWidget *widget, gpointer data )
+{
+  char *snapshot, *recording;
+
+  if( rzx_playback || rzx_recording ) return;
+
+  fuse_emulation_pause();
+
+  snapshot = gtkui_fileselector_get_filename( "Fuse - Replay Snapshot" );
+  if( !snapshot ) { fuse_emulation_unpause(); return; }
+  
+  recording = gtkui_fileselector_get_filename( "Fuse - Start Replay" );
+  if( !recording ) { free( snapshot ); fuse_emulation_unpause(); return; }
+
+  if( snapshot_read( snapshot ) ) {
+    free( snapshot ); free( recording );
+    return;
+  }
+
+  rzx_start_playback( recording );
+
+  free( snapshot ); free( recording );
+
+  display_refresh_all();
+
+  fuse_emulation_unpause();
+}
+
 /* Called by the menu when File/Exit selected */
 static void gtkui_quit(GtkWidget *widget, gpointer data)
 {
   fuse_exiting=1;
 }
 
-typedef struct gtkui_options_info {
-
-  GtkWidget *dialog;
-
-  GtkWidget *issue2;
-  GtkWidget *joy_kempston;
-  GtkWidget *tape_traps;
-  GtkWidget *stereo_ay;
-  GtkWidget *slt_traps;
-
-} gtkui_options_info;
-
-/* Called by the menu when Options/General selected */
-static void gtkui_options( GtkWidget *widget, gpointer data )
-{
-  gtkui_options_info dialog;
-  GtkWidget *ok_button, *cancel_button;
-  
-  /* Firstly, stop emulation */
-  fuse_emulation_pause();
-
-  /* Create the necessary widgets */
-  dialog.dialog = gtk_dialog_new();
-  gtk_window_set_title( GTK_WINDOW( dialog.dialog ),
-			"Fuse - General Options" );
-
-  dialog.issue2 =
-    gtk_check_button_new_with_label( "Issue 2 keyboard" );
-  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dialog.issue2 ),
-				settings_current.issue2 );
-
-  dialog.joy_kempston =
-    gtk_check_button_new_with_label( "Kempston joystick" );
-  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dialog.joy_kempston ),
-				settings_current.joy_kempston );
-
-  dialog.tape_traps =
-    gtk_check_button_new_with_label( "Use tape traps" );
-  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dialog.tape_traps ),
-				settings_current.tape_traps );
-
-  dialog.stereo_ay =
-    gtk_check_button_new_with_label( "AY stereo separation" );
-  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dialog.stereo_ay ),
-				settings_current.stereo_ay );
-
-  dialog.slt_traps =
-    gtk_check_button_new_with_label( "Use .slt traps" );
-  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( dialog.slt_traps ),
-				settings_current.slt_traps );
-
-  if( !fuse_sound_in_use )
-    gtk_widget_set_sensitive( dialog.stereo_ay, FALSE );
-
-  ok_button = gtk_button_new_with_label( "OK" );
-  cancel_button = gtk_button_new_with_label( "Cancel" );
-
-  /* Put everything into the dialog box */
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog.dialog )->vbox ),
-		     dialog.issue2 );
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog.dialog )->vbox ),
-		     dialog.joy_kempston );
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog.dialog )->vbox ),
-		     dialog.tape_traps );
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog.dialog )->vbox ),
-		     dialog.stereo_ay );
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog.dialog )->vbox ),
-		     dialog.slt_traps );
-
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog.dialog )->action_area ),
-		     ok_button );
-  gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog.dialog )->action_area ),
-		     cancel_button );
-
-  /* Add the necessary callbacks */
-  gtk_signal_connect( GTK_OBJECT( ok_button ), "clicked",
-		      GTK_SIGNAL_FUNC( gtkui_options_done ),
-		      (gpointer) &dialog );
-  gtk_signal_connect_object( GTK_OBJECT( cancel_button ), "clicked",
-			     GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
-			     GTK_OBJECT( dialog.dialog ) );
-  gtk_signal_connect( GTK_OBJECT( dialog.dialog ), "delete_event",
-		      GTK_SIGNAL_FUNC( gtkui_destroy_widget_and_quit ),
-		      (gpointer) NULL );
-
-  /* Allow Esc to cancel */
-  gtk_widget_add_accelerator( cancel_button, "clicked",
-                              gtk_accel_group_get_default(),
-                              GDK_Escape, 0, 0 );
-
-  /* Set the window to be modal and display it */
-  gtk_window_set_modal( GTK_WINDOW( dialog.dialog ), TRUE );
-  gtk_widget_show_all( dialog.dialog );
-
-  /* Process events until the window is done with */
-  gtk_main();
-
-  /* And then carry on with emulation again */
-  fuse_emulation_unpause();
-}
-
 /* Called by the menu when Machine/Reset selected */
 static void gtkui_reset(GtkWidget *widget, gpointer data)
 {
@@ -401,8 +395,7 @@ static void gtkui_select(GtkWidget *widget, gpointer data)
   /* Some space to store the radio buttons in */
   dialog.buttons = (GtkWidget**)malloc( machine_count * sizeof(GtkWidget* ) );
   if( dialog.buttons == NULL ) {
-    fprintf( stderr, "%s: out of memory at %s:%d\n",
-	     fuse_progname, __FILE__, __LINE__ );
+    ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
     return;
   }
 
@@ -495,7 +488,7 @@ static void gtkui_tape_open( GtkWidget *widget, gpointer data )
 
   fuse_emulation_pause();
 
-  filename = gtkui_fileselector_get_filename();
+  filename = gtkui_fileselector_get_filename( "Fuse - Open Tape" );
   if( !filename ) { fuse_emulation_unpause(); return; }
 
   tape_open( filename );
@@ -530,7 +523,7 @@ static void gtkui_tape_write( GtkWidget *widget, gpointer data )
 
   fuse_emulation_pause();
 
-  filename = gtkui_fileselector_get_filename();
+  filename = gtkui_fileselector_get_filename( "Fuse - Write Tape" );
   if( !filename ) { fuse_emulation_unpause(); return; }
 
   tape_write( filename );
@@ -548,7 +541,7 @@ static void gtkui_help_keyboard( GtkWidget *widget, gpointer data )
 }
 
 /* Generic `tidy-up' callback */
-static void gtkui_destroy_widget_and_quit( GtkWidget *widget, gpointer data )
+void gtkui_destroy_widget_and_quit( GtkWidget *widget, gpointer data )
 {
   gtk_widget_destroy( widget );
   gtk_main_quit();
@@ -563,11 +556,11 @@ typedef struct gktui_fileselector_info {
 
 } gtkui_fileselector_info;
 
-static char* gtkui_fileselector_get_filename( void )
+static char* gtkui_fileselector_get_filename( const char *title )
 {
   gtkui_fileselector_info selector;
 
-  selector.selector = gtk_file_selection_new( "Select File" );
+  selector.selector = gtk_file_selection_new( title );
   selector.filename = NULL;
 
   gtk_signal_connect(
@@ -627,29 +620,4 @@ static void gtkui_fileselector_cancel( GtkButton *button, gpointer user_data )
   gtk_main_quit();
 }
 
-/* Callbacks used by the Options dialog */
-static void gtkui_options_done( GtkWidget *widget, gpointer user_data )
-{
-  gtkui_options_info *ptr = (gtkui_options_info*) user_data;
-
-  settings_current.issue2 =
-    gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( ptr->issue2 ) );
-
-  settings_current.joy_kempston =
-    gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( ptr->joy_kempston ) );
-
-  settings_current.tape_traps =
-    gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( ptr->tape_traps ) );
-
-  settings_current.stereo_ay =
-    gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( ptr->stereo_ay ) );
-
-  settings_current.slt_traps =
-    gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( ptr->slt_traps ) );
-
-  gtk_widget_destroy( ptr->dialog );
-
-  gtk_main_quit();
-}
-
 #endif			/* #ifdef UI_GTK */
