@@ -1,7 +1,7 @@
 /* gtkui.h: GTK+ routines for dealing with the user interface
-   Copyright (c) 2000-2001 Philip Kendall
+   Copyright (c) 2000-2002 Philip Kendall, Russell Marks
 
-   $Id: gtkui.h,v 1.11 2001/08/13 21:49:40 pak21 Exp $
+   $Id: gtkui.h,v 1.15 2002/02/26 01:22:37 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,7 +31,15 @@
 #include <gtk/gtk.h>
 #endif
 
+#ifndef FUSE_GTKOPTIONS_H
+#include "options.h"
+#endif
+
 extern GtkWidget* gtkui_window;
 extern GtkWidget* gtkui_drawing_area;
 
+void gtkui_destroy_widget_and_quit( GtkWidget *widget, gpointer data );
+
+extern void gtkui_popup_menu(void);
+
 #endif			/* #ifndef FUSE_GTKUI_H */
