@@ -1,7 +1,7 @@
 /* ui.h: General UI event handling routines
-   Copyright (c) 2000-2001 Philip Kendall
+   Copyright (c) 2000-2002 Philip Kendall
 
-   $Id: ui.h,v 1.11 2001/08/13 21:49:40 pak21 Exp $
+   $Id: ui.h,v 1.13 2002/03/21 21:53:37 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,8 +27,17 @@
 #ifndef FUSE_UI_H
 #define FUSE_UI_H
 
+/* The various severities of error level, increasing downwards */
+typedef enum ui_error_level {
+
+  UI_ERROR_INFO,		/* Informational message */
+  UI_ERROR_ERROR,		/* An actual error */
+
+} ui_error_level;
+
 int ui_init(int *argc, char ***argv, int width, int height);
 int ui_event(void);
+int ui_error( ui_error_level severity, const char *format, ... );
 int ui_end(void);
 
 #endif			/* #ifndef FUSE_UI_H */
