@@ -1,7 +1,7 @@
 /* xdisplay.h: Routines for dealing with the X display
    Copyright (c) 2000 Philip Kendall
 
-   $Id: xdisplay.h,v 1.11 2001/08/13 21:49:40 pak21 Exp $
+   $Id: xdisplay.h,v 1.12 2002/03/09 23:10:48 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,4 +30,13 @@
 int xdisplay_configure_notify(int width, int height);
 void xdisplay_area(int x, int y, int width, int height);
 
+/* Are we expecting an X error to occur? */
+extern int xerror_expecting;
+
+/* Which error occured? */
+extern int xerror_error;
+
+/* The X error handler */
+int xerror_handler( Display *display, XErrorEvent *error );
+
 #endif			/* #ifndef FUSE_XDISPLAY_H */
