@@ -1,7 +1,7 @@
 /* utils.c: some useful helper functions
    Copyright (c) 1999-2002 Philip Kendall
 
-   $Id: utils.c,v 1.3 2002/01/15 21:33:50 pak21 Exp $
+   $Id: utils.c,v 1.7 2002/03/21 21:53:37 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,17 +26,19 @@
 
 #include <config.h>
 
+#include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <string.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <ui/ui.h>
 #include <unistd.h>
 
 #include "fuse.h"
 #include "utils.h"
 
-#define ERROR_MESSAGE_MAX_LENGTH 1024
 #define PATHNAME_MAX_LENGTH 1024
 
 /* Find a ROM called `filename'; look in the current directory, ./roms
@@ -64,13 +66,12 @@ int utils_read_file( const char *filename, unsigned char **buffer,
 {
   int fd;
 
-  int error; char error_message[ ERROR_MESSAGE_MAX_LENGTH ];
+  int error;
 
   fd = open( filename, O_RDONLY );
   if( fd == -1 ) {
-    snprintf( error_message, ERROR_MESSAGE_MAX_LENGTH,
-	      "%s: couldn't open `%s'", fuse_progname, filename );
-    perror( error_message );
+    ui_error( UI_ERROR_ERROR, "couldn't open '%s': %s", filename,
+	      strerror( errno ) );
     return 1;
   }
 
@@ -85,12 +86,9 @@ int utils_read_fd( int fd, const char *filename,
 {
   struct stat file_info;
 
-  char error_message[ ERROR_MESSAGE_MAX_LENGTH ];
-
   if( fstat( fd, &file_info) ) {
-    snprintf( error_message, ERROR_MESSAGE_MAX_LENGTH,
-	      "%s: Couldn't stat `%s'", fuse_progname, filename );
-    perror( error_message );
+    ui_error( UI_ERROR_ERROR, "Couldn't stat '%s': %s", filename,
+	      strerror( errno ) );
     close(fd);
     return 1;
   }
@@ -99,17 +97,15 @@ int utils_read_fd( int fd, const char *filename,
 
   (*buffer) = mmap( 0, *length, PROT_READ, MAP_SHARED, fd, 0 );
   if( *buffer == (void*)-1 ) {
-    snprintf( error_message, ERROR_MESSAGE_MAX_LENGTH,
-	      "%s: Couldn't mmap `%s'", fuse_progname, filename );
-    perror( error_message );
+    ui_error( UI_ERROR_ERROR, "Couldn't mmap '%s': %s", filename,
+	      strerror( errno ) );
     close(fd);
     return 1;
   }
 
   if( close(fd) ) {
-    snprintf( error_message, ERROR_MESSAGE_MAX_LENGTH,
-	      "%s: Couldn't close `%s'", fuse_progname, filename );
-    perror( error_message );
+    ui_error( UI_ERROR_ERROR, "Couldn't close '%s': %s", filename,
+	      strerror( errno ) );
     munmap( *buffer, *length );
     return 1;
   }
@@ -122,28 +118,23 @@ int utils_write_file( const char *filename, const unsigned char *buffer,
 {
   FILE *f;
 
-  char error_message[ ERROR_MESSAGE_MAX_LENGTH ];
-
   f=fopen( filename, "wb" );
   if(!f) { 
-    snprintf( error_message, ERROR_MESSAGE_MAX_LENGTH,
-	      "%s: error opening `%s'", fuse_progname, filename );
-    perror( error_message );
+    ui_error( UI_ERROR_ERROR, "error opening '%s': %s", filename,
+	      strerror( errno ) );
     return 1;
   }
 	    
   if( fwrite( buffer, 1, length, f ) != length ) {
-    snprintf( error_message, ERROR_MESSAGE_MAX_LENGTH,
-	      "%s: error writing to `%s'", fuse_progname, filename );
-    perror( error_message );
+    ui_error( UI_ERROR_ERROR, "error writing to '%s': %s", filename,
+	      strerror( errno ) );
     fclose(f);
     return 1;
   }
 
   if( fclose( f ) ) {
-    snprintf( error_message, ERROR_MESSAGE_MAX_LENGTH,
-	      "%s: error closing `%s'", fuse_progname, filename );
-    perror( error_message );
+    ui_error( UI_ERROR_ERROR, "error closing '%s': %s", filename,
+	      strerror( errno ) );
     return 1;
   }
 
