@@ -1,7 +1,7 @@
 /* z80_ops.c: Process the next opcode
    Copyright (c) 1999-2000,2002 Philip Kendall
 
-   $Id: z80_ops.c,v 1.18 2002/01/19 17:37:32 pak21 Exp $
+   $Id: z80_ops.c,v 1.20 2002/02/09 00:33:38 pak21 Exp $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <stdio.h>
 
 #include "event.h"
+#include "rzx.h"
 #include "settings.h"
 #include "spectrum.h"
 #include "tape.h"
@@ -52,6 +53,14 @@ void z80_do_opcodes()
 
     BYTE opcode;
 
+    /* If we're due an interrupt from RZX playback, generate one */
+    if( rzx_playback &&
+	rzx_instructions >= rzx.frames[ rzx_current_frame ].instructions ) {
+      event_add( tstates, EVENT_TYPE_INTERRUPT );
+      break;		/* And break out of the execution loop to let
+			   the interrupt happen */
+    }
+
     /* If the z80 is HALTed, execute a NOP-equivalent and loop again */
     if(z80.halted) {
       tstates+=4;
@@ -59,7 +68,7 @@ void z80_do_opcodes()
     }
 
     contend( PC, 4 );
-    opcode=readbyte(PC++); R++;
+    opcode=readbyte(PC++); R++; rzx_instructions++;
 
     switch(opcode) {
     case 0x00:		/* NOP */
@@ -839,7 +848,7 @@ void z80_do_opcodes()
 	BYTE opcode2;
 	contend( PC, 4 );
 	opcode2 = readbyte(PC++);
-	R++;
+	R++; rzx_instructions++;
 #ifdef HAVE_ENOUGH_MEMORY
 	switch(opcode2) {
 #include "z80_cb.c"
@@ -944,7 +953,7 @@ void z80_do_opcodes()
 	BYTE opcode2;
 	contend( PC, 4 );
 	opcode2 = readbyte(PC++);
-	R++;
+	R++; rzx_instructions++;
 #ifdef HAVE_ENOUGH_MEMORY
 	switch(opcode2) {
 #define REGISTER  IX
@@ -1038,7 +1047,7 @@ void z80_do_opcodes()
 	BYTE opcode2;
 	contend( PC, 4 );
 	opcode2 = readbyte(PC++);
-	R++;
+	R++; rzx_instructions++;
 #ifdef HAVE_ENOUGH_MEMORY
 	switch(opcode2) {
 #include "z80_ed.c"
@@ -1120,7 +1129,7 @@ void z80_do_opcodes()
 	BYTE opcode2;
 	contend( PC, 4 ); 
 	opcode2 = readbyte(PC++);
-	R++;
+	R++; rzx_instructions++;
 #ifdef HAVE_ENOUGH_MEMORY
 	switch(opcode2) {
 #define REGISTER  IY
